@@ -28,14 +28,14 @@
  * MT safe
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "config.h"
 
 #include <stdlib.h>
 #include <string.h>
-#include "glib.h"
+#include <signal.h>
 
+#include "glib.h"
+#include "gthreadinit.h"
 
 /* notes on macros:
  * having DISABLE_MEM_POOLS defined, disables mem_chunks alltogether, their
@@ -228,14 +228,13 @@ static gboolean vtable_set = FALSE;
  * 
  * Checks whether the allocator used by g_malloc() is the system's
  * malloc implementation. If it returns %TRUE memory allocated with
- * <function>malloc()</function> can be used interchangeable with 
- * memory allocated using g_malloc(). This function is useful for 
- * avoiding an extra copy of allocated memory returned by a 
- * non-GLib-based API.
+ * malloc() can be used interchangeable with memory allocated using g_malloc(). 
+ * This function is useful for avoiding an extra copy of allocated memory returned
+ * by a non-GLib-based API.
  *
  * A different allocator can be set using g_mem_set_vtable().
  *
- * Return value: if %TRUE, <function>malloc()</function> and g_malloc() can be mixed.
+ * Return value: if %TRUE, malloc() and g_malloc() can be mixed.
  **/
 gboolean
 g_mem_is_system_malloc (void)
@@ -540,7 +539,7 @@ profiler_try_realloc (gpointer mem,
   
   if (mem && p[0])	/* free count */
     {
-      g_warning ("realloc(%p, %u): memory has been freed %lu times already", p + 2, n_bytes, p[0]);
+      g_warning ("realloc(%p, %lu): memory has been freed %lu times already", p + 2, (gulong)n_bytes, p[0]);
       profiler_log (PROFILER_ALLOC | PROFILER_RELOC, n_bytes, FALSE);
 
       return NULL;
@@ -718,12 +717,12 @@ g_mem_chunk_destroy (GMemChunk *mem_chunk)
       g_free (temp_area);
     }
   
+  g_mutex_lock (mem_chunks_lock);
   if (mem_chunk->next)
     mem_chunk->next->prev = mem_chunk->prev;
   if (mem_chunk->prev)
     mem_chunk->prev->next = mem_chunk->next;
   
-  g_mutex_lock (mem_chunks_lock);
   if (mem_chunk == mem_chunks)
     mem_chunks = mem_chunks->next;
   g_mutex_unlock (mem_chunks_lock);
@@ -1020,8 +1019,10 @@ g_mem_chunk_reset (GMemChunk *mem_chunk)
   mem_chunk->free_atoms = NULL;
   
   if (mem_chunk->mem_tree)
-    g_tree_destroy (mem_chunk->mem_tree);
-  mem_chunk->mem_tree = g_tree_new ((GCompareFunc) g_mem_chunk_area_compare);
+    {
+      g_tree_destroy (mem_chunk->mem_tree);
+      mem_chunk->mem_tree = g_tree_new ((GCompareFunc) g_mem_chunk_area_compare);
+    }
 
   LEAVE_MEM_CHUNK_ROUTINE ();
 }
@@ -1043,7 +1044,7 @@ g_mem_chunk_print (GMemChunk *mem_chunk)
       mem_areas = mem_areas->next;
     }
 
-  g_log (g_log_domain_glib, G_LOG_LEVEL_INFO,
+  g_log (G_LOG_DOMAIN, G_LOG_LEVEL_INFO,
 	 "%s: %ld bytes using %d mem areas",
 	 mem_chunk->name, mem, mem_chunk->num_mem_areas);
 }
@@ -1064,7 +1065,7 @@ g_mem_chunk_info (void)
     }
   g_mutex_unlock (mem_chunks_lock);
   
-  g_log (g_log_domain_glib, G_LOG_LEVEL_INFO, "%d mem chunks", count);
+  g_log (G_LOG_DOMAIN, G_LOG_LEVEL_INFO, "%d mem chunks", count);
   
   g_mutex_lock (mem_chunks_lock);
   mem_chunk = mem_chunks;
@@ -1251,13 +1252,22 @@ g_allocator_free (GAllocator *allocator)
 }
 
 void
-g_mem_init (void)
+_g_mem_thread_init (void)
 {
 #ifndef DISABLE_MEM_POOLS
   mem_chunks_lock = g_mutex_new ();
 #endif
 #ifndef G_DISABLE_CHECKS
-  mem_chunk_recursion = g_private_new (NULL);
   g_profile_mutex = g_mutex_new ();
 #endif
 }
+
+void
+_g_mem_thread_private_init (void)
+{
+#ifndef G_DISABLE_CHECKS
+  g_assert (mem_chunk_recursion == NULL);
+  mem_chunk_recursion = g_private_new (NULL);
+#endif
+}
+
